@@ -129,7 +129,7 @@ localize_expression(char *dest, const char *src, int dest_length, int *cursor)
             g_string_append_unichar(output, g_utf8_get_char(c));
             
             /* Insert separator after nth digit */
-            if (v->display.show_tsep && v->base == DEC &&
+            if (v->display.show_tsep && v->base == 10 &&
                 !after_radix && digit_count > 1 && digit_count % v->tsep_count == 1) {
                 g_string_append(output, v->tsep);
                 if (new_cursor > read_cursor) {
@@ -171,7 +171,7 @@ display_clear(GCDisplay *display)
 }
 
 
-static const char *
+const char *
 display_get_text(GCDisplay *display)
 {
     return get_state(display)->expression;
@@ -253,7 +253,7 @@ display_set_number(GCDisplay *display, const MPNumber *MPval)
 void
 display_set_answer(GCDisplay *display)
 {
-    display_set_string(display, "Ans", -1);
+    display_set_string(display, "ans", -1);
 }
 
 
@@ -276,7 +276,7 @@ display_make_text(GCDisplay *display, char *localized, int length, int *cursor)
         
     /* Substitute answer register */
     display_make_number(display, temp, MAX_LOCALIZED, &e->ans, v->base, TRUE);
-    str = str_replace(str, "Ans", temp);
+    str = str_replace(str, "ans", temp);
 
     /* Replace registers with values. */
     for (i = 0; i < 10; i++) {
@@ -389,7 +389,7 @@ void display_push(GCDisplay *display)
         do {
             i = ((i + 1) % UNDO_HISTORY_LENGTH);
             free(display->h.e[i].expression);
-            display->h.e[i].expression = strdup("Ans");
+            display->h.e[i].expression = strdup("ans");
         } while (i != display->h.end);
     }
 
@@ -474,7 +474,7 @@ display_insert(GCDisplay *display, int cursor_start, int cursor_end, const char
                 use = FALSE;
             
             /* Ignore thousands separators */
-            if (strncmp(c, v->tsep, strlen(v->tsep)) == 0)
+            if (v->tsep[0] != '\0' && strncmp(c, v->tsep, strlen(v->tsep)) == 0)
                 use = FALSE;
             
             /* Copy existing text */
@@ -526,9 +526,9 @@ display_backspace(GCDisplay *display, int cursor_start, int cursor_end)
         
         len = g_utf8_strlen(ui_get_display(), -1);
         
-        if (exp_has_postfix(e->expression, "Ans")) {
+        if (exp_has_postfix(e->expression, "ans")) {
             display_make_number(display, buf, MAX_DISPLAY, &e->ans, v->base, FALSE);
-            e->expression = str_replace(e->expression, "Ans", buf);
+            e->expression = str_replace(e->expression, "ans", buf);
         } else {
             for (i = 0; i < 10; i++) {
                 SNPRINTF(buf, MAX_DISPLAY, "R%d", i);
@@ -540,7 +540,7 @@ display_backspace(GCDisplay *display, int cursor_start, int cursor_end)
                 }
             }
         }
-
+        
         display_insert(display, len - 1, len, "");
     } else if (cursor_start != cursor_end) {
         display_insert(display, cursor_start, cursor_end, "");
@@ -581,7 +581,7 @@ display_is_empty(GCDisplay *display)
 gboolean
 display_is_result(GCDisplay *display)
 {
-    if (strcmp(display_get_text(display), "Ans") == 0)
+    if (strcmp(display_get_text(display), "ans") == 0)
         return TRUE;
     
     return FALSE;
@@ -589,12 +589,13 @@ display_is_result(GCDisplay *display)
 
 
 gboolean
-display_is_usable_number(GCDisplay *display, MPNumber *MPnum)
+display_is_usable_number(GCDisplay *display, MPNumber *z)
 {
     if (display_is_empty(display)) {
-        return mp_equation_parse("0", MPnum);
+        mp_set_from_integer(0, z);
+        return TRUE;
     } else {
-        return mp_equation_parse(display_get_text(display), MPnum);
+        return display_solve(display, z) == 0; // FIXME: Change to MP
     }
 }
 
@@ -671,21 +672,28 @@ void display_set_format(GCDisplay *display, DisplayFormat type)
 }
 
 
+// FIXME: Obsolete
 int
 display_solve(GCDisplay *display, MPNumber *result)
 {
+    MPEquationOptions options;
     const char *text;
     int errorCode;
+    
+    memset(&options, 0, sizeof(options));
+    options.base = v->base;
+    options.wordlen = v->wordlen;
+    options.angle_units = v->ttype;
 
     text = display_get_text(display);
-    errorCode = mp_equation_parse(text, result);
+    errorCode = mp_equation_parse(text, &options, result);
     
     return errorCode;
 }
 
 
 /* Convert engineering or scientific number in the given base. */
-void
+static void
 make_eng_sci(GCDisplay *display, char *target, int target_len, const MPNumber *MPnumber, int base)
 {
     static char digits[] = "0123456789ABCDEF";   
@@ -710,10 +718,9 @@ make_eng_sci(GCDisplay *display, char *target, int target_len, const MPNumber *M
     }
     mp_set_from_mp(&MPval, &MPmant);
 
-    mp_set_from_integer(basevals[base], &MP1base);
-    mp_pwr_integer(&MP1base, 3, &MP3base);
-
-    mp_pwr_integer(&MP1base, 10, &MP10base);
+    mp_set_from_integer(base, &MP1base);
+    mp_xpowy_integer(&MP1base, 3, &MP3base);
+    mp_xpowy_integer(&MP1base, 10, &MP10base);
 
     mp_set_from_integer(1, &MP1);
     mp_divide(&MP1, &MP10base, &MPatmp);
@@ -743,7 +750,7 @@ make_eng_sci(GCDisplay *display, char *target, int target_len, const MPNumber *M
         }
     }
  
-    mp_cast_to_string(&MPmant, basevals[base], v->accuracy, !v->display.show_zeroes, fixed, MAX_DIGITS);
+    mp_cast_to_string(&MPmant, base, v->accuracy, !v->display.show_zeroes, fixed, MAX_DIGITS);
     len = strlen(fixed);
     for (i = 0; i < len; i++) {
         *optr++ = fixed[i];
@@ -785,14 +792,7 @@ make_eng_sci(GCDisplay *display, char *target, int target_len, const MPNumber *M
 void
 display_make_number(GCDisplay *display, char *target, int target_len, const MPNumber *MPnumber, int base, int ignoreError)
 {
-    static double max_fix[MAXBASES] = {
-       1.298074214e+33,    /* Binary. */
-       2.037035976e+90,    /* Octal. */
-       1.000000000e+100,   /* Decimal */
-       2.582249878e+120    /* Hexadecimal. */
-    };
-
-    double val;
+    double val, max_fix;
     
     /*  NOTE: display_make_number can currently set v->error when converting to a double.
      *        This is to provide the same look&feel as V3 even though gcalctool
@@ -809,12 +809,30 @@ display_make_number(GCDisplay *display, char *target, int target_len, const MPNu
         target[0] = '\0';
         return;
     }
+
+    switch (base)
+    {
+    case 2:
+        max_fix = 1.298074214e+33;
+        break;
+    case 8:
+        max_fix = 2.037035976e+90;
+        break;
+    case 10:
+        max_fix = 1.000000000e+100;
+        break;
+    default:
+    case 16:
+        max_fix = 2.582249878e+120;
+        break;
+    }
+    
     // FIXME: Do this based on the number of digits, not actual values
     if ((display->format == ENG) ||
         (display->format == SCI) ||
-        (display->format == FIX && val != 0.0 && (val > max_fix[base]))) {
+        (display->format == FIX && val != 0.0 && (val > max_fix))) {
         make_eng_sci(display, target, target_len, MPnumber, base);
     } else {
-        mp_cast_to_string(MPnumber, basevals[base], v->accuracy, !v->display.show_zeroes, target, target_len);
+        mp_cast_to_string(MPnumber, base, v->accuracy, !v->display.show_zeroes, target, target_len);
     }
 }
